@@ -30,7 +30,8 @@ sub decode_text {
   my $two_bit_code = 0;
   my $two_bit_flag = 0;
   # spec 3.4
-  my $flen = Games::Rezrov::StoryFile::header()->file_length();
+  my $zh = Games::Rezrov::StoryFile::header();
+  my $flen = $zh->file_length();
       
   while (1) {
     last if $address >= $flen;
@@ -51,48 +52,41 @@ sub decode_text {
 	  $two_bit_code = $two_bit_flag = 0;
 	  # done
 	}
-      } elsif ($abbreviation > 0) {
+      } elsif ($abbreviation) {
 	# synonym/abbreviation; spec 3.3
 	my $entry = (32 * ($abbreviation - 1)) + $zchar;
 #	print STDERR "abbrev $abbreviation\n";
-	my $addr = Games::Rezrov::StoryFile::header()->get_abbreviation_addr($entry);
+	my $addr = $zh->get_abbreviation_addr($entry);
 	$self->decode_text($addr, $buf_ref);
 	$abbreviation = 0;
-      } elsif ($zchar == 0) {
-#	$receiver->write_zchar(SPACE);
-	$$buf_ref .= " ";
-      } elsif ($zchar == 4) {
-	# spec 3.2.3: shift character; alphabet 1
-	$alphabet = 1;
-      } elsif ($zchar == 5) {
-	# spec 3.2.3: shift character; alphabet 2
-	$alphabet = 2;
-      } elsif ($zchar >= 1 && $zchar <= 3) {
-	# spec 3.3: next zchar is an abbreviation code
-	$abbreviation = $zchar;
+      } elsif ($zchar < 6) {
+	if ($zchar == 0) {
+	  #	$receiver->write_zchar(SPACE);
+	  $$buf_ref .= " ";
+	} elsif ($zchar == 4) {
+	  # spec 3.2.3: shift character; alphabet 1
+	  $alphabet = 1;
+	} elsif ($zchar == 5) {
+	  # spec 3.2.3: shift character; alphabet 2
+	  $alphabet = 2;
+	} elsif ($zchar >= 1 && $zchar <= 3) {
+	  # spec 3.3: next zchar is an abbreviation code
+	  $abbreviation = $zchar;
+	}
       } else {
 	# spec 3.5: convert remaining chars from alpha table
-	if ($zchar == 0) {
-#	  $receiver->write_zchar(SPACE);
-	  $$buf_ref .= " ";
+	$zchar -= 6;
+	# convert to string index
+	if ($alphabet < 2) {
+	  $$buf_ref .= $alpha_table[$alphabet]->[$zchar];
 	} else {
-	  $zchar -= 6;
-	  # convert to string index
-	  if ($alphabet < 2) {
-#	    $receiver->write_zchar(ord $alpha_table[$alphabet]->[$zchar]);
-	    $$buf_ref .= $alpha_table[$alphabet]->[$zchar];
-#	    die $$buf_ref;
+	  # alphabet 2; some special cases (3.5.3)
+	  if ($zchar == 0) {
+	    $two_bit_flag = 1;
+	  } elsif ($zchar == 1) {
+	    $$buf_ref .= chr(Games::Rezrov::ZConst::Z_NEWLINE());
 	  } else {
-	    # alphabet 2; some special cases (3.5.3)
-	    if ($zchar == 0) {
-	      $two_bit_flag = 1;
-	    } elsif ($zchar == 1) {
-#	      $receiver->write_zchar(Games::Rezrov::ZConst::Z_NEWLINE());
-	      $$buf_ref .= chr(Games::Rezrov::ZConst::Z_NEWLINE());
-	    } else {
-#	      $receiver->write_zchar(ord $alpha_table[$alphabet]->[$zchar]);
-	      $$buf_ref .= $alpha_table[$alphabet]->[$zchar];
-	    }
+	    $$buf_ref .= $alpha_table[$alphabet]->[$zchar];
 	  }
 	}
 	$alphabet = 0;
